"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const AWS_REGION = process.env.AWS_DEFAULT_REGION;
/**
 * Ref:
 * - https://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/DynamoDB/DocumentClient.html
 * - https://www.fernandomc.com/posts/eight-examples-of-fetching-data-from-dynamodb-with-node/
 */
class DynamoDbClientService {
    constructor(tableName) {
        if (!tableName)
            throw new Error('The "tableName" is required to use a DynamoDbClientService.');
        this.tableName = tableName;
        const client = new client_dynamodb_1.DynamoDBClient({ region: AWS_REGION });
        this.dynamoDBClient = lib_dynamodb_1.DynamoDBDocumentClient.from(client, {
            marshallOptions: {
                removeUndefinedValues: true,
            },
        });
    }
    /**
     * Get an item by key
     * @param {Object} keyObject - Ex: { jobId: 1234 }
     * @param {GetCommandInput} options
     */
    async getByKey(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    keyObject, options = {}) {
        const params = Object.assign({ TableName: this.tableName, Key: keyObject }, options);
        const result = await this.dynamoDBClient.send(new lib_dynamodb_1.GetCommand(params));
        return result.Item;
    }
    /**
     * Query a table
     * @param {QueryCommandInput} options
     */
    async query(options = {}) {
        const params = Object.assign({ TableName: this.tableName }, options);
        const result = await this.dynamoDBClient.send(new lib_dynamodb_1.QueryCommand(params));
        if (result.$metadata)
            delete result.$metadata;
        return result;
    }
    /**
     * Create/Replace a item object
     * To take care:
     * - https://stackoverflow.com/questions/43667229/difference-between-dynamodb-putitem-vs-updateitem
     * @param {Object} input
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async put(input) {
        const params = {
            TableName: this.tableName,
            Item: input,
            ReturnValues: 'NONE',
        };
        return await this.dynamoDBClient.send(new lib_dynamodb_1.PutCommand(params));
    }
    /**
     * Update a item object
     * To take care:
     * - https://stackoverflow.com/questions/43667229/difference-between-dynamodb-putitem-vs-updateitem
     * @param {Object} keyObject - Ex: { jobId: 1234 }
     * @param {UpdateCommandInput} updateObject
     */
    async update(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    keyObject, options = {}) {
        const params = Object.assign({ TableName: this.tableName, Key: keyObject, ReturnValues: 'NONE' }, options);
        return await this.dynamoDBClient.send(new lib_dynamodb_1.UpdateCommand(params));
    }
    /**
     * Delete/Remove an item object
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async remove(keyObject) {
        const params = {
            TableName: this.tableName,
            Key: keyObject,
            ReturnValues: 'NONE',
        };
        return await this.dynamoDBClient.send(new lib_dynamodb_1.DeleteCommand(params));
    }
}
exports.default = DynamoDbClientService;
//# sourceMappingURL=dynamoDbClient.service.js.map